@@ -1,14 +1,15 @@
+import re
 import glob
 import os
 import shutil
+from sys import version
 import tarfile
 from contextlib import contextmanager
 from operator import attrgetter
 from pathlib import Path
-from typing import Any, Dict, Iterable, List, Literal, Optional, Union, cast
+from typing import Any, Dict, List, Literal, Optional, Union, cast
 
 from loguru import logger
-from pytest import skip
 from pbt.config import PBTConfig
 from pbt.diff import Diff, diff_db
 from pbt.misc import ExecProcessError, cache_func, exec
@@ -227,22 +228,30 @@ class Poetry(PkgManager):
                     os.remove(pkg.location / "pyproject.toml.backup")
 
     def clean(self, pkg: Package):
-        exec("poetry env remove python", cwd=pkg.location, check_returncode=False)
+        exec(
+            "poetry env remove python",
+            cwd=pkg.location,
+            check_returncode=False,
+            **self.exec_options("env.remove"),
+        )
         for eggdir in glob.glob(str(pkg.location / "*.egg-info")):
             shutil.rmtree(eggdir)
         shutil.rmtree(pkg.location / "dist", ignore_errors=True)
 
     def publish(self, pkg: Package):
-        # exec("poetry publish --build", cwd=pkg.location, **self.exec_options("publish"))
-        raise NotImplementedError()
+        exec(
+            "poetry publish --build --no-interaction",
+            cwd=pkg.location,
+            **self.exec_options("publish"),
+        )
 
     def install(
         self,
         pkg: Package,
         editable: bool = False,
         include_dev: bool = False,
-        skip_deps: List[str] = None,
-        additional_deps: Dict[str, DepConstraints] = None,
+        skip_deps: Optional[List[str]] = None,
+        additional_deps: Optional[Dict[str, DepConstraints]] = None,
     ):
         skip_deps = skip_deps or []
         additional_deps = additional_deps or {}
@@ -273,14 +282,17 @@ class Poetry(PkgManager):
 
         if editable and pkg.name not in self.cfg.phantom_packages:
             self.build_editable(pkg, skip_deps=skip_deps)
-            exec([self.python_path(pkg), "setup.py", "develop"], cwd=pkg.location)
+            exec(
+                [self.python_path(pkg), "setup.py", "develop"],
+                cwd=pkg.location,
+            )
             (pkg.location / "setup.py").unlink()  # remove the setup.py file
 
     def build(
         self,
         pkg: Package,
-        skip_deps: List[str] = None,
-        additional_deps: Dict[str, DepConstraints] = None,
+        skip_deps: Optional[List[str]] = None,
+        additional_deps: Optional[Dict[str, DepConstraints]] = None,
     ):
         """Build the package. Support ignoring some dependencies to avoid installing and solving
         dependencies multiple times (not in the super interface as compiled languages will complain).
@@ -308,7 +320,11 @@ class Poetry(PkgManager):
                     try:
                         if (pkg.location / "dist").exists():
                             shutil.rmtree(str(pkg.location / "dist"))
-                        exec("poetry build", cwd=pkg.location)
+                        exec(
+                            "poetry build",
+                            cwd=pkg.location,
+                            **self.exec_options("build"),
+                        )
                     finally:
                         diff.save(db)
 
@@ -317,7 +333,7 @@ class Poetry(PkgManager):
     def build_editable(
         self,
         pkg: Package,
-        skip_deps: List[str],
+        skip_deps: Optional[List[str]] = None,
     ):
         """Build egg package that can be used to install in editable mode as poetry does not
         provide it out of the box.
@@ -341,6 +357,11 @@ class Poetry(PkgManager):
                 assert memberfile is not None
                 f.write(memberfile.read())
 
+            exec(
+                [self.python_path(pkg), "setup.py", "bdist_egg"],
+                cwd=pkg.location,
+            )
+
     def get_fixed_version_pkgs(self):
         return self.fixed_version_pkgs
 
@@ -349,7 +370,7 @@ class Poetry(PkgManager):
         self.build(pkg)
         whl_path = self.wheel_path(pkg)
         assert whl_path is not None
-        output = exec(["pip", "hash", whl_path])[1]
+        output = exec([self.pip_path(pkg), "hash", whl_path])[1]
         output = output[output.find("--hash=") + len("--hash=") :]
         assert output.startswith("sha256:")
         return output[len("sha256:") :]
@@ -359,7 +380,7 @@ class Poetry(PkgManager):
         pkg: Package,
         dependency: Package,
         editable: bool = False,
-        skip_dep_deps: List[str] = None,
+        skip_dep_deps: Optional[List[str]] = None,
     ):
         assert dependency.name not in self.cfg.phantom_packages, dependency.name
         skip_dep_deps = skip_dep_deps or []
@@ -400,15 +421,19 @@ class Poetry(PkgManager):
 
         if len(output) > 1:
             logger.warning(
-                "There are multiple virtual environments for package {}, and we pick the first one (maybe incorrect)",
+                "There are multiple virtual environments for package {}, and we pick the first one (maybe incorrect). List of environments: \n{}",
                 name,
+                "\n".join(["\t- " + x for x in output]),
             )
 
+        assert len(output) > 0
         if output[0].endswith(" (Activated)"):
             path = output[0].split(" ")[0]
         else:
             path = output[0]
 
+        if not Path(path).exists():
+            raise Exception(f"Environment path {path} doesn't exist")
         return Path(path)
 
     def tar_path(self, pkg: Package) -> Optional[Path]:
@@ -432,29 +457,47 @@ class Poetry(PkgManager):
         return self.env_path(pkg.name, pkg.location) / "bin/python"  # type: ignore
 
     def exec_options(
-        self, cmd: Literal["publish", "install", "env.create", "env.fetch"]
+        self,
+        cmd: Literal[
+            "publish",
+            "install",
+            "build",
+            "env.remove",
+            "env.create",
+            "env.fetch",
+        ],
     ) -> dict:
-        if cmd in {"publish", "install"}:
-            return {}
-        return {}
+        return {"env": ["PATH"]}
 
     def parse_dep_spec(self, spec: Union[str, dict]) -> DepConstraint:
         if isinstance(spec, str):
-            return DepConstraint(version_spec=spec)
+            constraint = f"python=* markers="
+            return DepConstraint(version_spec=spec, constraint=constraint)
         elif isinstance(spec, dict):
             if "version" not in spec:
-                raise NotImplementedError(
-                    f"Not support specify dependency outside of Pypi yet. But found spec {spec}"
-                )
+                if "url" in spec:
+                    # try to figure out the version from the URL if possible, otherwise, use 1.0.0
+                    m = re.search(r"\d+.\d+.\d+", spec["url"])
+                    if m is not None:
+                        version_spec = f"=={m.group()}"
+                    else:
+                        version_spec = "==1.0.0"
+                else:
+                    raise NotImplementedError(
+                        f"Not support specify dependency outside of Pypi yet. But found spec {spec}"
+                    )
+            else:
+                version_spec = spec["version"]
 
             constraint = (
                 f"python={spec.get('python', '*')} markers={spec.get('markers', '')}"
             )
             origin_spec = spec.copy()
-            origin_spec.pop("version")
+            if "version" in origin_spec:
+                origin_spec.pop("version")
 
             return DepConstraint(
-                version_spec=spec["version"],
+                version_spec=version_spec,
                 constraint=constraint,
                 version_spec_field="version",
                 origin_spec=origin_spec,
